Struct FileId
pub struct FileId(pub u32);Expand description
A file identifier used to track which source file a span belongs to.
File IDs are indices into a file table maintained by the compiler.
FileId(0) is reserved as the default/unknown file.
Tuple Fields
0: u32Implementations